"""
Silique (https://www.silique.fr)
Copyright (C) 2024-2025

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
        
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
            
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from rougail.error import DictConsistencyError
from rougail.annotator.variable import Walk
from .i18n import _


class Annotator(Walk):
    """Annotate for environment"""
                
    level = 95
                
    def __init__(
        self,
        objectspace,
        *args,  # pylint: disable=unused-argument
    ) -> None:
        if not objectspace.paths:
            return
        self.objectspace = objectspace
        self.check_family()
        self.check_variable()

    def check_family(self):
        for family in self.get_families():
            if family.name != family.name.lower():
                msg = _('family name must be a lowercase name when we want to use user data "environment", so "{0}" is invalid')
                raise DictConsistencyError(msg.format(family.name), 200, family.xmlfiles)

    def check_variable(self):
        for variable in self.get_variables():
            if variable.name != variable.name.lower():
                msg = _('variable name must be a lowercase name when we want to use user data "environment", so "{0}" is invalid')
                raise DictConsistencyError(msg.format(variable.name), 201, variable.xmlfiles)
